@@ -11,6 +11,7 @@ use fs_err as fs;
 use fs_err::File;
 use ignore::overrides::Override;
 use ignore::WalkBuilder;
+use indexmap::IndexMap;
 use normpath::PathExt as _;
 use sha2::{Digest, Sha256};
 use std::collections::{HashMap, HashSet};
@@ -93,7 +94,7 @@ impl PathWriter {
                     anyhow!("Expected `python` to be a python interpreter inside a virtualenv ಠ_ಠ")
                 })?;
 
-        let base_path = target.get_venv_site_package(venv_dir, &interpreter);
+        let base_path = interpreter.get_venv_site_package(venv_dir, target);
 
         Ok(PathWriter {
             base_path,
@@ -506,7 +507,7 @@ Root-Is-Purelib: false
 /// https://packaging.python.org/specifications/entry-points/
 fn entry_points_txt(
     entry_type: &str,
-    entrypoints: &HashMap<String, String, impl std::hash::BuildHasher>,
+    entrypoints: &IndexMap<String, String, impl std::hash::BuildHasher>,
 ) -> String {
     entrypoints
         .iter()
@@ -709,7 +710,6 @@ fn handle_cffi_call_result(
 pub fn write_bindings_module(
     writer: &mut impl ModuleWriter,
     project_layout: &ProjectLayout,
-    module_name: &str,
     artifact: &Path,
     python_interpreter: Option<&PythonInterpreter>,
     is_abi3: bool,
@@ -762,25 +762,23 @@ pub fn write_bindings_module(
             writer.add_file_with_permissions(relative.join(&so_filename), artifact, 0o755)?;
         }
     } else {
-        let module = PathBuf::from(module_name);
+        let module = PathBuf::from(ext_name);
         writer.add_directory(&module)?;
         // Reexport the shared library as if it were the top level module
         writer.add_bytes(
             &module.join("__init__.py"),
             format!(
-                r#"from .{module_name} import *
+                r#"from .{ext_name} import *
 
-__doc__ = {module_name}.__doc__
-if hasattr({module_name}, "__all__"):
-    __all__ = {module_name}.__all__"#
+__doc__ = {ext_name}.__doc__
+if hasattr({ext_name}, "__all__"):
+    __all__ = {ext_name}.__all__"#
             )
             .as_bytes(),
         )?;
-        let type_stub = project_layout
-            .rust_module
-            .join(format!("{module_name}.pyi"));
+        let type_stub = project_layout.rust_module.join(format!("{ext_name}.pyi"));
         if type_stub.exists() {
-            eprintln!("📖 Found type stub file at {module_name}.pyi");
+            eprintln!("📖 Found type stub file at {ext_name}.pyi");
             writer.add_file(&module.join("__init__.pyi"), type_stub)?;
             writer.add_bytes(&module.join("py.typed"), b"")?;
         }
@@ -896,40 +894,38 @@ fn generate_uniffi_bindings(
         );
     }
 
+    let mut cmd = Command::new("uniffi-bindgen");
+    cmd.args([
+        "generate",
+        "--no-format",
+        "--language",
+        "python",
+        "--out-dir",
+    ]);
+    cmd.arg(&binding_dir);
+
     let udl = &udls[0];
     let config_file = crate_dir.join("uniffi.toml");
     let mut cdylib_name = None;
-    let config_file = {
-        if config_file.is_file() {
-            let uniffi_toml: UniFfiToml =
-                toml_edit::easy::from_str(&fs::read_to_string(&config_file)?)?;
-            cdylib_name = uniffi_toml
-                .bindings
-                .get("python")
-                .and_then(|py| py.cdylib_name.clone());
-            Some(
-                config_file
-                    .as_path()
-                    .try_into()
-                    .expect("path contains non-utf8"),
-            )
-        } else {
-            None
-        }
-    };
-    uniffi_bindgen::generate_bindings(
-        udl.as_path().try_into().expect("path contains non-utf8"),
-        config_file,
-        vec!["python"],
-        Some(
-            binding_dir
-                .as_path()
-                .try_into()
-                .expect("path contains non-utf8"),
-        ),
-        None,
-        false,
+    if config_file.is_file() {
+        let uniffi_toml: UniFfiToml = toml::from_str(&fs::read_to_string(&config_file)?)?;
+        cdylib_name = uniffi_toml
+            .bindings
+            .get("python")
+            .and_then(|py| py.cdylib_name.clone());
+        cmd.arg("--config");
+        cmd.arg(config_file);
+    }
+    cmd.arg(udl);
+    debug!("Running {:?}", cmd);
+    let mut child = cmd.spawn().context(
+        "Failed to run uniffi-bindgen, did you install it? Try `pip install uniffi-bindgen`",
     )?;
+    let exit_status = child.wait().context("Failed to run uniffi-bindgen")?;
+    if !exit_status.success() {
+        bail!("Command {:?} failed", cmd);
+    }
+
     let py_binding_name = udl.file_stem().unwrap();
     let py_binding = binding_dir.join(py_binding_name).with_extension("py");
     let name = py_binding_name.to_str().unwrap().to_string();
@@ -1153,7 +1149,7 @@ pub fn write_python_part(
         if let Some(glob_patterns) = pyproject.include() {
             for pattern in glob_patterns
                 .iter()
-                .filter_map(|glob_pattern| glob_pattern.targets(Format::Sdist))
+                .filter_map(|glob_pattern| glob_pattern.targets(Format::Wheel))
             {
                 eprintln!("📦 Including files matching \"{pattern}\"");
                 for source in glob::glob(&pyproject_dir.join(pattern).to_string_lossy())
@@ -1277,13 +1273,14 @@ pub fn add_data(writer: &mut impl ModuleWriter, data: Option<&Path>) -> Result<(
 #[cfg(test)]
 mod tests {
     use ignore::overrides::OverrideBuilder;
+    use pep440_rs::Version;
 
     use super::*;
 
     #[test]
     // The mechanism is the same for wheel_writer
     fn sdist_writer_excludes() -> Result<(), Box<dyn std::error::Error>> {
-        let metadata = Metadata21::default();
+        let metadata = Metadata21::new("dummy".to_string(), Version::from_release(vec![1, 0]));
         let perm = 0o777;
 
         // No excludes
