@@ -119,17 +119,20 @@ impl ProjectResolver {
 
         // If the package name contains minuses, you must declare a module with
         // underscores as lib name
-        let module_name = cargo_toml
-            .lib
-            .as_ref()
-            .and_then(|lib| lib.name.as_ref())
+        // Precedence:
+        //  * Explicitly declared pyproject.toml `tool.maturin.module-name`
+        //  * Cargo.toml `lib.name`
+        //  * pyproject.toml `project.name`
+        //  * Cargo.toml `package.name`
+        let module_name = pyproject
+            .and_then(|x| x.module_name())
+            .or(cargo_toml.lib.as_ref().and_then(|lib| lib.name.as_deref()))
+            .or(pyproject
+                .and_then(|pyproject| pyproject.project.as_ref())
+                .map(|project| project.name.as_str()))
             .unwrap_or(crate_name)
             .to_owned();
 
-        let extension_name = pyproject
-            .and_then(|x| x.module_name())
-            .unwrap_or(&module_name);
-
         let project_root = if pyproject_file.is_file() {
             pyproject_file.parent().unwrap_or(manifest_dir)
         } else {
@@ -175,8 +178,15 @@ impl ProjectResolver {
                 project_root.join(data)
             }
         });
-        let project_layout =
-            ProjectLayout::determine(project_root, extension_name, py_root, python_packages, data)?;
+        let custom_python_source = pyproject.and_then(|x| x.python_source()).is_some();
+        let project_layout = ProjectLayout::determine(
+            project_root,
+            &module_name,
+            py_root,
+            python_packages,
+            data,
+            custom_python_source,
+        )?;
         Ok(Self {
             project_layout,
             cargo_toml_path: manifest_file,
@@ -333,15 +343,15 @@ impl ProjectResolver {
 impl ProjectLayout {
     /// Checks whether a python module exists besides Cargo.toml with the right name
     fn determine(
-        project_root: impl AsRef<Path>,
+        project_root: &Path,
         module_name: &str,
         python_root: PathBuf,
         python_packages: Vec<String>,
         data: Option<PathBuf>,
+        custom_python_source: bool,
     ) -> Result<ProjectLayout> {
         // A dot in the module name means the extension module goes into the module folder specified by the path
         let parts: Vec<&str> = module_name.split('.').collect();
-        let project_root = project_root.as_ref();
         let (python_module, rust_module, extension_name) = if parts.len() > 1 {
             let mut rust_module = python_root.clone();
             rust_module.extend(&parts[0..parts.len() - 1]);
@@ -363,6 +373,7 @@ impl ProjectLayout {
             rust_module = %rust_module.display(),
             python_module = %python_module.display(),
             extension_name = %extension_name,
+            module_name = %module_name,
             "Project layout resolved"
         );
 
@@ -395,6 +406,15 @@ impl ProjectLayout {
                 data,
             })
         } else {
+            if custom_python_source {
+                eprintln!(
+                    "⚠️ Warning: You specified the python source as {}, but the python module at \
+                    {} is missing. No python module will be included.",
+                    python_root.display(),
+                    python_module.display()
+                );
+            }
+
             Ok(ProjectLayout {
                 python_dir: python_root,
                 python_packages,
